@@ -1,7 +1,7 @@
 /*
     MIT License
 
-    Copyright (c) 2019 Marius Appel <marius.appel@uni-muenster.de>
+    Copyright (c) 2020 Marius Appel <marius.appel@uni-muenster.de>
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to deal
@@ -25,14 +25,16 @@
 #define IMAGE_COLLECTION_CUBE_H
 
 #include <unordered_set>
+
 #include "cube.h"
+#include "image_collection.h"
 
 namespace gdalcubes {
 
 struct image_mask {
     virtual ~image_mask() {}
     virtual void apply(double *mask_buf, double *pixel_buf, uint32_t nb, uint32_t ny, uint32_t nx) = 0;
-    virtual nlohmann::json as_json() = 0;
+    virtual json11::Json as_json() = 0;
 };
 
 struct value_mask : public image_mask {
@@ -72,8 +74,8 @@ struct value_mask : public image_mask {
         }
     }
 
-    nlohmann::json as_json() override {
-        nlohmann::json out;
+    json11::Json as_json() override {
+        json11::Json::object out;
         out["mask_type"] = "value_mask";
         out["values"] = _mask_values;
         out["invert"] = _invert;
@@ -125,8 +127,8 @@ struct range_mask : public image_mask {
         }
     }
 
-    nlohmann::json as_json() override {
-        nlohmann::json out;
+    json11::Json as_json() override {
+        json11::Json::object out;
         out["mask_type"] = "range_mask";
         out["min"] = _min;
         out["max"] = _max;
@@ -276,12 +278,6 @@ class image_collection_cube : public cube {
         GCBS_ERROR("Band '" + band + "' does not exist in image collection, image mask will not be modified.");
     }
 
-    // set additional GDAL warp arguments like whether or not to use overviews, how GCPs should be interpolated, or
-    // performance settings
-    void set_warp_args(std::vector<std::string> args) {
-        _warp_args = args;  // TODO: do some checks that users do not overwrite settings like -of, -r, -tr, -ts, -te, -s_srs, -t_srs, -ot, -wt
-    }
-
     std::shared_ptr<chunk_data> read_chunk(chunkid_t id) override;
 
     // image_collection_cube is the only class that supports changing chunk sizes from outside!
@@ -290,46 +286,25 @@ class image_collection_cube : public cube {
         _chunk_size = {t, y, x};
     }
 
-    nlohmann::json make_constructible_json() override {
+    json11::Json make_constructible_json() override {
         if (_collection->is_temporary()) {
             throw std::string("ERROR in image_collection_cube::make_constructible_json(): image collection is temporary, please export as file using write() first.");
         }
-        nlohmann::json out;
+        json11::Json::object out;
         out["cube_type"] = "image_collection";
-        out["chunk_size"] = {_chunk_size[0], _chunk_size[1], _chunk_size[2]};
-        out["view"] = nlohmann::json::parse(std::dynamic_pointer_cast<cube_view>(_st_ref)->write_json_string());
+        out["chunk_size"] = json11::Json::array({(int)_chunk_size[0], (int)_chunk_size[1], (int)_chunk_size[2]});
+        std::string err;  // TODO: do something with err
+        out["view"] = json11::Json::parse(std::dynamic_pointer_cast<cube_view>(_st_ref)->write_json_string(), err);
         out["file"] = _collection->get_filename();
         if (_mask) {
             out["mask"] = _mask->as_json();
             out["mask_band"] = _mask_band;
         }
-        out["warp_args"] = _warp_args;
         return out;
     }
 
     static cube_view default_view(std::shared_ptr<image_collection> ic);
 
-   protected:
-    virtual void set_st_reference(std::shared_ptr<cube_st_reference> stref) override {
-        // _st_ref has always type cube_view, whereas stref can have types st_reference or cube_view
-
-        // copy fields from st_reference type
-        _st_ref->win() = stref->win();
-        _st_ref->srs() = stref->srs();
-        _st_ref->ny() = stref->ny();
-        _st_ref->nx() = stref->nx();
-        _st_ref->t0() = stref->t0();
-        _st_ref->t1() = stref->t1();
-        _st_ref->dt(stref->dt());
-
-        // if view: copy aggregation and resampling
-        std::shared_ptr<cube_view> v = std::dynamic_pointer_cast<cube_view>(stref);
-        if (v) {
-            std::dynamic_pointer_cast<cube_view>(_st_ref)->aggregation_method() = v->aggregation_method();
-            std::dynamic_pointer_cast<cube_view>(_st_ref)->resampling_method() = v->resampling_method();
-        }
-    }
-
    private:
     const std::shared_ptr<image_collection> _collection;
 
@@ -339,7 +314,6 @@ class image_collection_cube : public cube {
 
     std::shared_ptr<image_mask> _mask;
     std::string _mask_band;
-    std::vector<std::string> _warp_args;
 };
 
 }  // namespace gdalcubes
