@@ -36,8 +36,12 @@ std::shared_ptr<chunk_data> join_bands_cube::read_chunk(chunkid_t id) {
     coords_nd<uint32_t, 4> size_btyx = {_bands.count(), size_tyx[0], size_tyx[1], size_tyx[2]};
     out->size(size_btyx);
 
-    std::shared_ptr<chunk_data> dat_A = _in_A->read_chunk(id);
-    std::shared_ptr<chunk_data> dat_B = _in_B->read_chunk(id);
+    //
+    //    std::shared_ptr<chunk_data> dat_A = _in_A->read_chunk(id);
+    //    std::shared_ptr<chunk_data> dat_B = _in_B->read_chunk(id);
+    //    if (dat_A->empty() && dat_B->empty()) {
+    //        return out;
+    //    }
 
     // Fill buffers accordingly
     out->buf(std::calloc(size_btyx[0] * size_btyx[1] * size_btyx[2] * size_btyx[3], sizeof(double)));
@@ -45,9 +49,20 @@ std::shared_ptr<chunk_data> join_bands_cube::read_chunk(chunkid_t id) {
     double *end = ((double *)out->buf()) + size_btyx[0] * size_btyx[1] * size_btyx[2] * size_btyx[3];
     std::fill(begin, end, NAN);
 
-    memcpy(((double *)out->buf()), ((double *)dat_A->buf()), dat_A->size()[0] * dat_A->size()[1] * dat_A->size()[2] * dat_A->size()[3] * sizeof(double));
-    memcpy(((double *)out->buf()) + dat_A->size()[0] * dat_A->size()[1] * dat_A->size()[2] * dat_A->size()[3], ((double *)dat_B->buf()), dat_B->size()[0] * dat_B->size()[1] * dat_B->size()[2] * dat_B->size()[3] * sizeof(double));
-
+    uint32_t offset = 0;
+    bool allempty = true;
+    for (uint16_t i = 0; i < _in.size(); ++i) {
+        std::shared_ptr<chunk_data> dat = _in[i]->read_chunk(id);
+        if (!dat->empty()) {
+            allempty = false;
+            std::memcpy(((double *)out->buf()) + offset, ((double *)dat->buf()), dat->size()[0] * dat->size()[1] * dat->size()[2] * dat->size()[3] * sizeof(double));
+        }
+        offset += _in[i]->size_bands() * size_tyx[0] * size_tyx[1] * size_tyx[2];
+    }
+    if (allempty) {
+        // propagated empty chunk if all input chunks are emtpy
+        out = std::make_shared<chunk_data>();
+    }
     return out;
 }
 
